library IEEE;
use IEEE.std_logic_1164.all;

entity teiler is
port( 
  clk : in std_logic;
  reset : in std_logic;
  clock_out : out std_logic
);
end teiler;

architecture behavior of teiler is
signal count : integer range 0 to 20000000;

begin
teiler: process (clk, reset)
begin
  if reset = '1' then
    count <= 0;
    clock_out <= '0';
  else
    if clk = '1' and clk'event then
      if count < 10500000 then
        clock_out <= '0';
      else
        clock_out <= '1';
      end if;
      if count > 20000000 then
        count <= 0;
      end if;
      count <= count + 1;
    end if;
  end if;
end process;
end behavior;

library ieee;
use ieee.std_logic_1164.all;

entity ampel is
generic (
  k : positive := 10
); 

port ( 
  clk : in std_logic;
  reset : in std_logic;
  fussg_button : in std_logic;
  gruen_1 : out std_logic;
  gelb_1 : out std_logic;
  rot_1 : out std_logic;
  gruen_2 : out std_logic;
  gelb_2 : out std_logic;
  rot_2 : out std_logic;
  gruen_fussg : out std_logic;
  rot_fussg : out std_logic
);
end ampel;

architecture state_machine of ampel is
component teiler
port ( 
  clk : in std_logic;
  reset : in std_logic;
  clock_out : out std_logic
);
end component;

type sreg0_type is (s0,s1,s2,s3,s4,s5,s6,s7,s8); 
signal sreg0 : sreg0_type;

signal fussg, fussg_button_old : std_logic; 
signal clock  : std_logic; 
signal count1 : integer range 0 to 31; 
signal clear, rst, fussg_off, fussg_rst: std_logic;

begin
  rst       <= (reset or clear);
  fussg_rst <=(reset or fussg_off);

ampelteiler: teiler
port map ( 
  clk       => clk,
  reset     => reset,
  clock_out => clock
);

fussg_check: process (clk, fussg_rst)
begin
  if fussg_rst='1' then
    fussg<='0';
  else
    if rising_edge(clk) then
      if fussg_button = '0' and fussg_button_old = '1' then
        fussg<='1';
      end if;
      fussg_button_old<=fussg_button;
    end if;
  end if;
end process fussg_check;

machine: process (reset, clock)
begin
  if reset='1' then 
    sreg0<=s0;
  else
    if rising_edge(clock) then 
      case sreg0 is 
      when s0 =>
        if count1=k then 
          sreg0<=s1;
        else
          sreg0<=s0;
        end if;
      when s1 => -- rot gelb rot
        sreg0<=s2;
      when s2 => -- rot rot rot
        sreg0<=s3;
      when s3 => -- rot/gelb rot rot
        sreg0<=s4;
      when s4 => -- gruen rot rot
        if count1=k then 
          sreg0<=s5;
        else
          sreg0<=s4;
        end if;
      when s5 => -- gelb rot rot
        sreg0<=s6;
      when s6 => -- rot rot rot
        if fussg='1' then 
          sreg0<=s7;
        else
          sreg0<=s8;
        end if;
      when s7 => -- rot rot gruen
        if count1=k then 
          sreg0<=s6;
        else
          sreg0<=s7;
        end if;
      when s8 => -- rot rot/gelb rot
        sreg0<=s0;
      when others =>
        sreg0<=s0;
      end case;
    end if;
  end if;
end process machine;

counter: process (clock,rst)
begin
   if rst='1' then 
      count1<=0;
   else
     if rising_edge(clock) then 
      count1<=count1+1; 
    end if;
  end if;
end process counter;

  gruen_1_assignment:
  gruen_1 <= '0' when (sreg0 = s0) else
  '0' when (sreg0 = s1) else
  '0' when (sreg0 = s2) else
  '0' when (sreg0 = s3) else
  '1' when (sreg0 = s4) else
  '0' when (sreg0 = s5) else
  '0' when (sreg0 = s6) else
  '0' when (sreg0 = s7) else
  '0' when (sreg0 = s8) else
  '0';
  
  gelb_1_assignment:
  gelb_1 <= '0' when (sreg0 = s0) else
  '0' when (sreg0 = s1) else
  '0' when (sreg0 = s2) else
  '1' when (sreg0 = s3) else
  '0' when (sreg0 = s4) else
  '1' when (sreg0 = s5) else
  '0' when (sreg0 = s6) else
  '0' when (sreg0 = s7) else
  '0' when (sreg0 = s8) else
  '0';
  
  rot_1_assignment:
  rot_1 <= '1' when (sreg0 = s0) else
  '1' when (sreg0 = s1) else
  '1' when (sreg0 = s2) else
  '1' when (sreg0 = s3) else
  '0' when (sreg0 = s4) else
  '0' when (sreg0 = s5) else
  '1' when (sreg0 = s6) else
  '1' when (sreg0 = s7) else
  '1' when (sreg0 = s8) else
  '0';
  
  gruen_2_assignment:
  gruen_2 <='1' when (sreg0 = s0) else
  '0' when (sreg0 = s1) else
  '0' when (sreg0 = s2) else
  '0' when (sreg0 = s3) else
  '0' when (sreg0 = s4) else
  '0' when (sreg0 = s5) else
  '0' when (sreg0 = s6) else
  '0' when (sreg0 = s7) else
  '0' when (sreg0 = s8) else
  '0';
  
  gelb_2_assignment:
  gelb_2 <= '0' when (sreg0 = s0) else
  '1' when (sreg0 = s1) else
  '0' when (sreg0 = s2) else
  '0' when (sreg0 = s3) else
  '0' when (sreg0 = s4) else
  '0' when (sreg0 = s5) else
  '0' when (sreg0 = s6) else
  '0' when (sreg0 = s7) else
  '1' when (sreg0 = s8) else
  '0';
  
  rot_2_assignment:
  rot_2 <= '0' when (sreg0 = s0) else
  '0' when (sreg0 = s1) else
  '1' when (sreg0 = s2) else
  '1' when (sreg0 = s3) else
  '1' when (sreg0 = s4) else
  '1' when (sreg0 = s5) else
  '1' when (sreg0 = s6) else
  '1' when (sreg0 = s7) else
  '1' when (sreg0 = s8) else
  '0';
  
  gruen_fussg_assignment:
  gruen_fussg <= '0' when (sreg0 = s0) else
  '0' when (sreg0 = s1) else
  '0' when (sreg0 = s2) else
  '0' when (sreg0 = s3) else
  '0' when (sreg0 = s4) else
  '0' when (sreg0 = s5) else
  '0' when (sreg0 = s6) else
  '1' when (sreg0 = s7) else
  '0' when (sreg0 = s8) else
  '0';
  
  rot_fussg_assignment:
  rot_fussg <= '1' when (sreg0 = s0) else
  '1' when (sreg0 = s1) else
  '1' when (sreg0 = s2) else
  '1' when (sreg0 = s3) else
  '1' when (sreg0 = s4) else
  '1' when (sreg0 = s5) else
  '1' when (sreg0 = s6) else
  '0' when (sreg0 = s7) else
  '1' when (sreg0 = s8) else
  '0';

  clear_assignment:
  clear <= '0' when (sreg0=s0) else
  '1' when (sreg0 = s1) else
  '1' when (sreg0 = s2) else
  '1' when (sreg0 = s3) else
  '0' when (sreg0 = s4) else
  '1' when (sreg0 = s5) else
  '1' when (sreg0 = s6) else
  '0' when (sreg0 = s7) else
  '1' when (sreg0 = s8) else
  '0';

  fussg_off_assignment:
  fussg_off<='1' when (sreg0=s7) else
  '0';
end state_machine;



