library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity vga_ampel is
port(
  clock : in std_logic;
  hsync : out std_logic;
  vsync : out std_logic;
  red   : OUT STD_LOGIC_VECTOR(3 downto 0);
  green : OUT STD_LOGIC_VECTOR(3 downto 0);
  blue  : OUT STD_LOGIC_VECTOR(3 downto 0);
  reset : in std_logic;
  fussg_button : in std_logic
  ); 
end vga_ampel;

architecture behavioral of vga_ampel is

signal offs     : unsigned (6 downto 0);
signal counterx : unsigned (9 downto 0);
signal countery : unsigned (9 downto 0);
signal basis    : unsigned (11 downto 0);
signal charlin  : unsigned (11 downto 0);
signal zpos     : unsigned (11 downto 0);
signal wr_addr  : unsigned (11 downto 0):="000110010000";  
signal dat      : std_logic_vector (7 downto 0);
signal char     : std_logic_vector (7 downto 0);
signal dat_w    : std_logic_vector (7 downto 0);

signal counterxmaxed : std_logic;
signal counterymaxed : std_logic;
signal h_enable      : std_logic;
signal v_enable      : std_logic;
signal vid_enable    : std_logic;
signal pixel         : std_logic;
signal clk25         : std_logic;
signal we            : std_logic;

signal g_ampel : std_logic_vector (7 downto 0);

signal clk     : std_logic;
signal gruen_1 : std_logic;
signal gelb_1  : std_logic;
signal rot_1   : std_logic;
signal gruen_2 : std_logic;
signal gelb_2  : std_logic;
signal rot_2   : std_logic;
signal gruen_fussg :  std_logic;
signal rot_fussg   :  std_logic;

component ampel port(
  clk          : in std_logic;
  reset        : in std_logic;
  fussg_button : in std_logic;
  gruen_1      : out std_logic;
  gelb_1       : out std_logic;
  rot_1        : out std_logic;
  gruen_2      : out std_logic;
  gelb_2       : out std_logic;
  rot_2        : out std_logic;
  gruen_fussg  : out std_logic;
  rot_fussg    : out std_logic
);  
end component;
  
component vga_maschine port(
  clk25 : in std_logic;
  hsync : out std_logic;
  vsync : out std_logic;
  
  offs     : inout unsigned (6 downto 0);
  counterx : inout unsigned (9 downto 0);
  countery : inout unsigned (9 downto 0);
  basis    : inout unsigned (11 downto 0);
  charlin  : out unsigned (11 downto 0);
  zpos     : out unsigned (11 downto 0); 
  
  dat      : in std_logic_vector (7 downto 0);
  char     : in std_logic_vector (7 downto 0);
  
  counterxmaxed : inout std_logic;
  counterymaxed : inout std_logic;
  h_enable      : inout std_logic;
  v_enable      : inout std_logic;
  vid_enable    : out std_logic;
  pixel         : out std_logic  
);  
end component;

component vga_pixel port(
  clock : in std_logic;
  red   : OUT STD_LOGIC_VECTOR(3 downto 0);
  green : OUT STD_LOGIC_VECTOR(3 downto 0);
  blue  : OUT STD_LOGIC_VECTOR(3 downto 0);
  
  g_ampel    : inout std_logic_vector (7 downto 0);
  vid_enable : inout std_logic;
  pixel      : inout std_logic;
  counterx   : inout unsigned (9 downto 0);
  countery   : inout unsigned (9 downto 0)
 );
end component;

component ram port(
  clk25   : in  std_logic;
  we      : in  std_logic;
  wr_addr : in  unsigned (11 downto 0);
  rd_addr : in  unsigned (11 downto 0);
  d       : in  std_logic_vector (7 downto 0);
  q       : out std_logic_vector (7 downto 0)
  );
end component;

component rom port(
  clk25  : in  std_logic;
  addr_a : in  unsigned (11 downto 0);
  q_a    : out std_logic_vector (7 downto 0)
  );
end component;

begin
io5 : ampel port map(
  clk     => clock,
  reset   => reset,
  gruen_1 => gruen_1,
  gelb_1  => gelb_1,
  rot_1   => rot_1,
  gruen_2 => gruen_2,
  gelb_2  => gelb_2,
  rot_2   => rot_2,
  rot_fussg    => rot_fussg,
  gruen_fussg  => gruen_fussg,
  fussg_button => fussg_button
 
);  

io4 : vga_maschine port map(
  clk25         => clk25,
  hsync         => hsync,
  vsync         => vsync,
  offs          => offs,
  counterx      => counterx,
  countery      => countery,
  basis         => basis,
  charlin       => charlin,
  zpos          => zpos,
  dat           => dat,
  char          => char,
  counterxmaxed => counterxmaxed,
  counterymaxed =>counterymaxed,
  h_enable      => h_enable,
  v_enable      => v_enable,
  vid_enable    => vid_enable,
  pixel         => pixel
);
  
io3 : vga_pixel port map(
 clock      => clock,
 g_ampel    => g_ampel,
 red        => red,
 green      => green,
 blue       => blue,
 vid_enable => vid_enable,
 pixel      => pixel,
 counterx   => counterx,
 countery   => countery
 );
  
io2 : ram  port map(
  clk25   => clk25,
  we      => we,
  rd_addr => zpos,
  wr_addr => wr_addr,
  q       => char,
  d       => dat_w
);

io1 : rom  port map(
  clk25  => clk25,
  addr_a => charlin,
  q_a    => dat
);

process (clock) begin
  if rising_edge(clock) then
    clk25 <= not clk25;
  end if;
end process;

process(clock) begin
  if rising_edge(clock) then 
    g_ampel<=rot_1 & gelb_1 & gruen_1 & rot_2 & gelb_2 & gruen_2 & rot_fussg & gruen_fussg;
  end if;  
end process;  

end behavioral;